+{
   entry => <<'ENTRY', # for DUCET v5.2.0
00F1      ; [.136E.0020.0002.00F1] # LATIN SMALL LETTER N WITH TILDE
006E 0303 ; [.136E.0020.0002.00F1] # LATIN SMALL LETTER N WITH TILDE
00D1      ; [.136E.0020.0008.00D1] # LATIN CAPITAL LETTER N WITH TILDE
004E 0303 ; [.136E.0020.0008.00D1] # LATIN CAPITAL LETTER N WITH TILDE
006E 0067 ; [.136F.0020.0002.006E] # <LATIN SMALL LETTER N, LATIN SMALL LETTER G>
004E 0067 ; [.136F.0020.0007.004E] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER G>
004E 0047 ; [.136F.0020.0008.004E] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER G>
ENTRY
};
