$EX_OK = 0x0;
$EX__BASE = 0x40;
$EX_USAGE = 0x40;
$EX_DATAERR = 0x41;
$EX_NOINPUT = 0x42;
$EX_NOUSER = 0x43;
$EX_NOHOST = 0x44;
$EX_UNAVAILABLE = 0x45;
$EX_SOFTWARE = 0x46;
$EX_OSERR = 0x47;
$EX_OSFILE = 0x48;
$EX_CANTCREAT = 0x49;
$EX_IOERR = 0x4A;
$EX_TEMPFAIL = 0x4B;
$EX_PROTOCOL = 0x4C;
$EX_NOPERM = 0x4D;
