+{
   entry => <<'ENTRY', # for DUCET v5.2.0
0063 0068 ; [.123E.0020.0002.0063] # <LATIN SMALL LETTER C, LATIN SMALL LETTER H>
0043 0068 ; [.123E.0020.0007.0043] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER H>
0043 0048 ; [.123E.0020.0008.0043] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER H>
006C 006C ; [.1331.0020.0002.006C] # <LATIN SMALL LETTER L, LATIN SMALL LETTER L>
004C 006C ; [.1331.0020.0007.004C] # <LATIN CAPITAL LETTER L, LATIN SMALL LETTER L>
004C 004C ; [.1331.0020.0008.004C] # <LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER L>
00F1      ; [.136E.0020.0002.00F1] # LATIN SMALL LETTER N WITH TILDE
006E 0303 ; [.136E.0020.0002.00F1] # LATIN SMALL LETTER N WITH TILDE
00D1      ; [.136E.0020.0008.00D1] # LATIN CAPITAL LETTER N WITH TILDE
004E 0303 ; [.136E.0020.0008.00D1] # LATIN CAPITAL LETTER N WITH TILDE
ENTRY
};
