+{
   entry => <<'ENTRY', # for DUCET v5.2.0
010D      ; [.123E.0020.0002.010D] # LATIN SMALL LETTER C WITH CARON
0063 030C ; [.123E.0020.0002.010D] # LATIN SMALL LETTER C WITH CARON
010C      ; [.123E.0020.0008.010C] # LATIN CAPITAL LETTER C WITH CARON
0043 030C ; [.123E.0020.0008.010C] # LATIN CAPITAL LETTER C WITH CARON
0107      ; [.123F.0020.0002.0107] # LATIN SMALL LETTER C WITH ACUTE
0063 0301 ; [.123F.0020.0002.0107] # LATIN SMALL LETTER C WITH ACUTE
0063 0341 ; [.123F.0020.0002.0107] # LATIN SMALL LETTER C WITH ACUTE
0106      ; [.123F.0020.0008.0106] # LATIN CAPITAL LETTER C WITH ACUTE
0043 0301 ; [.123F.0020.0008.0106] # LATIN CAPITAL LETTER C WITH ACUTE
0043 0341 ; [.123F.0020.0008.0106] # LATIN CAPITAL LETTER C WITH ACUTE
01C6      ; [.1251.0020.0002.01C6] # LATIN SMALL LETTER DZ WITH CARON
0064 017D ; [.1251.0020.0007.0064][.0000.0000.0002.0000] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z WITH CARON>
01C5      ; [.1251.0020.0007.01C5][.0000.0000.0008.0000] # LATIN CAPITAL LETTER D WITH SMALL LETTER Z WITH CARON
01C4      ; [.1251.0020.0008.01C4] # LATIN CAPITAL LETTER DZ WITH CARON
0064 017E ; [.1251.0020.0002.01C6] # <LATIN SMALL LETTER D, LATIN SMALL LETTER Z WITH CARON>
0044 017E ; [.1251.0020.0007.01C5][.0000.0000.0008.0000] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z WITH CARON>
0044 017D ; [.1251.0020.0008.01C4] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z WITH CARON>
0064 007A 030C ; [.1251.0020.0002.01C6] # <LATIN SMALL LETTER D, LATIN SMALL LETTER Z, COMBINING CARON>
0064 005A 030C ; [.1251.0020.0007.0064][.0000.0000.0002.0000] # <LATIN SMALL LETTER D, LATIN CAPITAL LETTER Z, COMBINING CARON>
0044 007A 030C ; [.1251.0020.0007.01C5][.0000.0000.0008.0000] # <LATIN CAPITAL LETTER D, LATIN SMALL LETTER Z, COMBINING CARON>
0044 005A 030C ; [.1251.0020.0008.01C4] # <LATIN CAPITAL LETTER D, LATIN CAPITAL LETTER Z, COMBINING CARON>
0111      ; [.1252.0020.0002.0111] # LATIN SMALL LETTER D WITH STROKE
0064 0335 ; [.1252.0020.0002.0111] # LATIN SMALL LETTER D WITH STROKE
0110      ; [.1252.0020.0008.0110] # LATIN CAPITAL LETTER D WITH STROKE
0044 0335 ; [.1252.0020.0008.0110] # LATIN CAPITAL LETTER D WITH STROKE
01C9      ; [.1331.0020.0002.01C9] # LATIN SMALL LETTER LJ
006C 006A ; [.1331.0020.0002.01C9] # <LATIN SMALL LETTER L, LATIN SMALL LETTER J>
006C 004A ; [.1331.0020.0007.006C][.0000.0000.0002.0000] # <LATIN SMALL LETTER L, LATIN CAPITAL LETTER J>
01C8      ; [.1331.0020.0007.01C8][.0000.0000.0008.0000] # LATIN CAPITAL LETTER L WITH SMALL LETTER J
004C 006A ; [.1331.0020.0007.01C8][.0000.0000.0008.0000] # <LATIN CAPITAL LETTER L, LATIN SMALL LETTER J>
01C7      ; [.1331.0020.0008.01C7] # LATIN CAPITAL LETTER LJ
004C 004A ; [.1331.0020.0008.01C7] # <LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER J>
01CC      ; [.136E.0020.0002.01CC] # LATIN SMALL LETTER NJ
006E 006A ; [.136E.0020.0002.01CC] # <LATIN SMALL LETTER N, LATIN SMALL LETTER J>
006E 004A ; [.136E.0020.0007.006E][.0000.0000.0002.0000] # <LATIN SMALL LETTER N, LATIN CAPITAL LETTER J>
01CB      ; [.136E.0020.0007.01CB][.0000.0000.0008.0000] # LATIN CAPITAL LETTER N WITH SMALL LETTER J
004E 006A ; [.136E.0020.0007.01CB][.0000.0000.0008.0000] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER J>
01CA      ; [.136E.0020.0008.01CA] # LATIN CAPITAL LETTER NJ
004E 004A ; [.136E.0020.0008.01CA] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER J>
0161      ; [.1411.0020.0002.0161] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.1411.0020.0002.0161] # LATIN SMALL LETTER S WITH CARON
0160      ; [.1411.0020.0008.0160] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.1411.0020.0008.0160] # LATIN CAPITAL LETTER S WITH CARON
017E      ; [.14AE.0020.0002.017E] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.14AE.0020.0002.017E] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.14AE.0020.0008.017D] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.14AE.0020.0008.017D] # LATIN CAPITAL LETTER Z WITH CARON
ENTRY
};
