/* -*- buffer-read-only: t -*-
 *
 *    opcode.h
 *
 *    Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
 *    2001, 2002, 2003, 2004, 2005, 2006, 2007 by Larry Wall and others
 *
 *    You may distribute under the terms of either the GNU General Public
 *    License or the Artistic License, as specified in the README file.
 *
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 *  This file is built by regen/opcode.pl from its data.  Any changes made
 *  here will be lost!
 */

#ifndef PERL_GLOBAL_STRUCT_INIT

#define Perl_pp_i_preinc Perl_pp_preinc
#define Perl_pp_i_predec Perl_pp_predec
#define Perl_pp_i_postinc Perl_pp_postinc
#define Perl_pp_i_postdec Perl_pp_postdec

PERL_PPDEF(Perl_unimplemented_op)

START_EXTERN_C

#define OP_NAME(o) ((o)->op_type == OP_CUSTOM ? custom_op_name(o) : \
                    PL_op_name[(o)->op_type])
#define OP_DESC(o) ((o)->op_type == OP_CUSTOM ? custom_op_desc(o) : \
                    PL_op_desc[(o)->op_type])

#ifndef DOINIT
EXTCONST char* const PL_op_name[];
#else
EXTCONST char* const PL_op_name[] = {
	"null",
	"stub",
	"scalar",
	"pushmark",
	"wantarray",
	"const",
	"gvsv",
	"gv",
	"gelem",
	"padsv",
	"padav",
	"padhv",
	"padany",
	"pushre",
	"rv2gv",
	"rv2sv",
	"av2arylen",
	"rv2cv",
	"anoncode",
	"prototype",
	"refgen",
	"srefgen",
	"ref",
	"bless",
	"backtick",
	"glob",
	"readline",
	"rcatline",
	"regcmaybe",
	"regcreset",
	"regcomp",
	"match",
	"qr",
	"subst",
	"substcont",
	"trans",
	"sassign",
	"aassign",
	"chop",
	"schop",
	"chomp",
	"schomp",
	"defined",
	"undef",
	"study",
	"pos",
	"preinc",
	"i_preinc",
	"predec",
	"i_predec",
	"postinc",
	"i_postinc",
	"postdec",
	"i_postdec",
	"pow",
	"multiply",
	"i_multiply",
	"divide",
	"i_divide",
	"modulo",
	"i_modulo",
	"repeat",
	"add",
	"i_add",
	"subtract",
	"i_subtract",
	"concat",
	"stringify",
	"left_shift",
	"right_shift",
	"lt",
	"i_lt",
	"gt",
	"i_gt",
	"le",
	"i_le",
	"ge",
	"i_ge",
	"eq",
	"i_eq",
	"ne",
	"i_ne",
	"ncmp",
	"i_ncmp",
	"slt",
	"sgt",
	"sle",
	"sge",
	"seq",
	"sne",
	"scmp",
	"bit_and",
	"bit_xor",
	"bit_or",
	"negate",
	"i_negate",
	"not",
	"complement",
	"smartmatch",
	"atan2",
	"sin",
	"cos",
	"rand",
	"srand",
	"exp",
	"log",
	"sqrt",
	"int",
	"hex",
	"oct",
	"abs",
	"length",
	"substr",
	"vec",
	"index",
	"rindex",
	"sprintf",
	"formline",
	"ord",
	"chr",
	"crypt",
	"ucfirst",
	"lcfirst",
	"uc",
	"lc",
	"quotemeta",
	"rv2av",
	"aelemfast",
	"aelem",
	"aslice",
	"aeach",
	"akeys",
	"avalues",
	"each",
	"values",
	"keys",
	"delete",
	"exists",
	"rv2hv",
	"helem",
	"hslice",
	"boolkeys",
	"unpack",
	"pack",
	"split",
	"join",
	"list",
	"lslice",
	"anonlist",
	"anonhash",
	"splice",
	"push",
	"pop",
	"shift",
	"unshift",
	"sort",
	"reverse",
	"grepstart",
	"grepwhile",
	"mapstart",
	"mapwhile",
	"range",
	"flip",
	"flop",
	"and",
	"or",
	"xor",
	"dor",
	"cond_expr",
	"andassign",
	"orassign",
	"dorassign",
	"method",
	"entersub",
	"leavesub",
	"leavesublv",
	"caller",
	"warn",
	"die",
	"reset",
	"lineseq",
	"nextstate",
	"dbstate",
	"unstack",
	"enter",
	"leave",
	"scope",
	"enteriter",
	"iter",
	"enterloop",
	"leaveloop",
	"return",
	"last",
	"next",
	"redo",
	"dump",
	"goto",
	"exit",
	"method_named",
	"entergiven",
	"leavegiven",
	"enterwhen",
	"leavewhen",
	"break",
	"continue",
	"open",
	"close",
	"pipe_op",
	"fileno",
	"umask",
	"binmode",
	"tie",
	"untie",
	"tied",
	"dbmopen",
	"dbmclose",
	"sselect",
	"select",
	"getc",
	"read",
	"enterwrite",
	"leavewrite",
	"prtf",
	"print",
	"say",
	"sysopen",
	"sysseek",
	"sysread",
	"syswrite",
	"eof",
	"tell",
	"seek",
	"truncate",
	"fcntl",
	"ioctl",
	"flock",
	"send",
	"recv",
	"socket",
	"sockpair",
	"bind",
	"connect",
	"listen",
	"accept",
	"shutdown",
	"gsockopt",
	"ssockopt",
	"getsockname",
	"getpeername",
	"lstat",
	"stat",
	"ftrread",
	"ftrwrite",
	"ftrexec",
	"fteread",
	"ftewrite",
	"fteexec",
	"ftis",
	"ftsize",
	"ftmtime",
	"ftatime",
	"ftctime",
	"ftrowned",
	"fteowned",
	"ftzero",
	"ftsock",
	"ftchr",
	"ftblk",
	"ftfile",
	"ftdir",
	"ftpipe",
	"ftsuid",
	"ftsgid",
	"ftsvtx",
	"ftlink",
	"fttty",
	"fttext",
	"ftbinary",
	"chdir",
	"chown",
	"chroot",
	"unlink",
	"chmod",
	"utime",
	"rename",
	"link",
	"symlink",
	"readlink",
	"mkdir",
	"rmdir",
	"open_dir",
	"readdir",
	"telldir",
	"seekdir",
	"rewinddir",
	"closedir",
	"fork",
	"wait",
	"waitpid",
	"system",
	"exec",
	"kill",
	"getppid",
	"getpgrp",
	"setpgrp",
	"getpriority",
	"setpriority",
	"time",
	"tms",
	"localtime",
	"gmtime",
	"alarm",
	"sleep",
	"shmget",
	"shmctl",
	"shmread",
	"shmwrite",
	"msgget",
	"msgctl",
	"msgsnd",
	"msgrcv",
	"semop",
	"semget",
	"semctl",
	"require",
	"dofile",
	"hintseval",
	"entereval",
	"leaveeval",
	"entertry",
	"leavetry",
	"ghbyname",
	"ghbyaddr",
	"ghostent",
	"gnbyname",
	"gnbyaddr",
	"gnetent",
	"gpbyname",
	"gpbynumber",
	"gprotoent",
	"gsbyname",
	"gsbyport",
	"gservent",
	"shostent",
	"snetent",
	"sprotoent",
	"sservent",
	"ehostent",
	"enetent",
	"eprotoent",
	"eservent",
	"gpwnam",
	"gpwuid",
	"gpwent",
	"spwent",
	"epwent",
	"ggrnam",
	"ggrgid",
	"ggrent",
	"sgrent",
	"egrent",
	"getlogin",
	"syscall",
	"lock",
	"once",
	"custom",
};
#endif

#ifndef DOINIT
EXTCONST char* const PL_op_desc[];
#else
EXTCONST char* const PL_op_desc[] = {
	"null operation",
	"stub",
	"scalar",
	"pushmark",
	"wantarray",
	"constant item",
	"scalar variable",
	"glob value",
	"glob elem",
	"private variable",
	"private array",
	"private hash",
	"private value",
	"push regexp",
	"ref-to-glob cast",
	"scalar dereference",
	"array length",
	"subroutine dereference",
	"anonymous subroutine",
	"subroutine prototype",
	"reference constructor",
	"single ref constructor",
	"reference-type operator",
	"bless",
	"quoted execution (``, qx)",
	"glob",
	"<HANDLE>",
	"append I/O operator",
	"regexp internal guard",
	"regexp internal reset",
	"regexp compilation",
	"pattern match (m//)",
	"pattern quote (qr//)",
	"substitution (s///)",
	"substitution iterator",
	"transliteration (tr///)",
	"scalar assignment",
	"list assignment",
	"chop",
	"scalar chop",
	"chomp",
	"scalar chomp",
	"defined operator",
	"undef operator",
	"study",
	"match position",
	"preincrement (++)",
	"integer preincrement (++)",
	"predecrement (--)",
	"integer predecrement (--)",
	"postincrement (++)",
	"integer postincrement (++)",
	"postdecrement (--)",
	"integer postdecrement (--)",
	"exponentiation (**)",
	"multiplication (*)",
	"integer multiplication (*)",
	"division (/)",
	"integer division (/)",
	"modulus (%)",
	"integer modulus (%)",
	"repeat (x)",
	"addition (+)",
	"integer addition (+)",
	"subtraction (-)",
	"integer subtraction (-)",
	"concatenation (.) or string",
	"string",
	"left bitshift (<<)",
	"right bitshift (>>)",
	"numeric lt (<)",
	"integer lt (<)",
	"numeric gt (>)",
	"integer gt (>)",
	"numeric le (<=)",
	"integer le (<=)",
	"numeric ge (>=)",
	"integer ge (>=)",
	"numeric eq (==)",
	"integer eq (==)",
	"numeric ne (!=)",
	"integer ne (!=)",
	"numeric comparison (<=>)",
	"integer comparison (<=>)",
	"string lt",
	"string gt",
	"string le",
	"string ge",
	"string eq",
	"string ne",
	"string comparison (cmp)",
	"bitwise and (&)",
	"bitwise xor (^)",
	"bitwise or (|)",
	"negation (-)",
	"integer negation (-)",
	"not",
	"1's complement (~)",
	"smart match",
	"atan2",
	"sin",
	"cos",
	"rand",
	"srand",
	"exp",
	"log",
	"sqrt",
	"int",
	"hex",
	"oct",
	"abs",
	"length",
	"substr",
	"vec",
	"index",
	"rindex",
	"sprintf",
	"formline",
	"ord",
	"chr",
	"crypt",
	"ucfirst",
	"lcfirst",
	"uc",
	"lc",
	"quotemeta",
	"array dereference",
	"constant array element",
	"array element",
	"array slice",
	"each on array",
	"keys on array",
	"values on array",
	"each",
	"values",
	"keys",
	"delete",
	"exists",
	"hash dereference",
	"hash element",
	"hash slice",
	"boolkeys",
	"unpack",
	"pack",
	"split",
	"join or string",
	"list",
	"list slice",
	"anonymous list ([])",
	"anonymous hash ({})",
	"splice",
	"push",
	"pop",
	"shift",
	"unshift",
	"sort",
	"reverse",
	"grep",
	"grep iterator",
	"map",
	"map iterator",
	"flipflop",
	"range (or flip)",
	"range (or flop)",
	"logical and (&&)",
	"logical or (||)",
	"logical xor",
	"defined or (//)",
	"conditional expression",
	"logical and assignment (&&=)",
	"logical or assignment (||=)",
	"defined or assignment (//=)",
	"method lookup",
	"subroutine entry",
	"subroutine exit",
	"lvalue subroutine return",
	"caller",
	"warn",
	"die",
	"symbol reset",
	"line sequence",
	"next statement",
	"debug next statement",
	"iteration finalizer",
	"block entry",
	"block exit",
	"block",
	"foreach loop entry",
	"foreach loop iterator",
	"loop entry",
	"loop exit",
	"return",
	"last",
	"next",
	"redo",
	"dump",
	"goto",
	"exit",
	"method with known name",
	"given()",
	"leave given block",
	"when()",
	"leave when block",
	"break",
	"continue",
	"open",
	"close",
	"pipe",
	"fileno",
	"umask",
	"binmode",
	"tie",
	"untie",
	"tied",
	"dbmopen",
	"dbmclose",
	"select system call",
	"select",
	"getc",
	"read",
	"write",
	"write exit",
	"printf",
	"print",
	"say",
	"sysopen",
	"sysseek",
	"sysread",
	"syswrite",
	"eof",
	"tell",
	"seek",
	"truncate",
	"fcntl",
	"ioctl",
	"flock",
	"send",
	"recv",
	"socket",
	"socketpair",
	"bind",
	"connect",
	"listen",
	"accept",
	"shutdown",
	"getsockopt",
	"setsockopt",
	"getsockname",
	"getpeername",
	"lstat",
	"stat",
	"-R",
	"-W",
	"-X",
	"-r",
	"-w",
	"-x",
	"-e",
	"-s",
	"-M",
	"-A",
	"-C",
	"-O",
	"-o",
	"-z",
	"-S",
	"-c",
	"-b",
	"-f",
	"-d",
	"-p",
	"-u",
	"-g",
	"-k",
	"-l",
	"-t",
	"-T",
	"-B",
	"chdir",
	"chown",
	"chroot",
	"unlink",
	"chmod",
	"utime",
	"rename",
	"link",
	"symlink",
	"readlink",
	"mkdir",
	"rmdir",
	"opendir",
	"readdir",
	"telldir",
	"seekdir",
	"rewinddir",
	"closedir",
	"fork",
	"wait",
	"waitpid",
	"system",
	"exec",
	"kill",
	"getppid",
	"getpgrp",
	"setpgrp",
	"getpriority",
	"setpriority",
	"time",
	"times",
	"localtime",
	"gmtime",
	"alarm",
	"sleep",
	"shmget",
	"shmctl",
	"shmread",
	"shmwrite",
	"msgget",
	"msgctl",
	"msgsnd",
	"msgrcv",
	"semop",
	"semget",
	"semctl",
	"require",
	"do \"file\"",
	"eval hints",
	"eval \"string\"",
	"eval \"string\" exit",
	"eval {block}",
	"eval {block} exit",
	"gethostbyname",
	"gethostbyaddr",
	"gethostent",
	"getnetbyname",
	"getnetbyaddr",
	"getnetent",
	"getprotobyname",
	"getprotobynumber",
	"getprotoent",
	"getservbyname",
	"getservbyport",
	"getservent",
	"sethostent",
	"setnetent",
	"setprotoent",
	"setservent",
	"endhostent",
	"endnetent",
	"endprotoent",
	"endservent",
	"getpwnam",
	"getpwuid",
	"getpwent",
	"setpwent",
	"endpwent",
	"getgrnam",
	"getgrgid",
	"getgrent",
	"setgrent",
	"endgrent",
	"getlogin",
	"syscall",
	"lock",
	"once",
	"unknown custom operator",
};
#endif

END_EXTERN_C

#endif /* !PERL_GLOBAL_STRUCT_INIT */

START_EXTERN_C

#ifdef PERL_GLOBAL_STRUCT_INIT
#  define PERL_PPADDR_INITED
static const Perl_ppaddr_t Gppaddr[]
#else
#  ifndef PERL_GLOBAL_STRUCT
#    define PERL_PPADDR_INITED
EXT Perl_ppaddr_t PL_ppaddr[] /* or perlvars.h */
#  endif
#endif /* PERL_GLOBAL_STRUCT */
#if (defined(DOINIT) && !defined(PERL_GLOBAL_STRUCT)) || defined(PERL_GLOBAL_STRUCT_INIT)
#  define PERL_PPADDR_INITED
= {
	Perl_pp_null,
	Perl_pp_stub,
	Perl_pp_null,	/* Perl_pp_scalar */
	Perl_pp_pushmark,
	Perl_pp_wantarray,
	Perl_pp_const,
	Perl_pp_gvsv,
	Perl_pp_gv,
	Perl_pp_gelem,
	Perl_pp_padsv,
	Perl_pp_padav,
	Perl_pp_padhv,
	Perl_unimplemented_op,	/* Perl_pp_padany */
	Perl_pp_pushre,
	Perl_pp_rv2gv,
	Perl_pp_rv2sv,
	Perl_pp_av2arylen,
	Perl_pp_rv2cv,
	Perl_pp_anoncode,
	Perl_pp_prototype,
	Perl_pp_refgen,
	Perl_pp_srefgen,
	Perl_pp_ref,
	Perl_pp_bless,
	Perl_pp_backtick,
	Perl_pp_glob,
	Perl_pp_readline,
	Perl_pp_rcatline,
	Perl_pp_null,	/* Perl_pp_regcmaybe */
	Perl_pp_regcreset,
	Perl_pp_regcomp,
	Perl_pp_match,
	Perl_pp_qr,
	Perl_pp_subst,
	Perl_pp_substcont,
	Perl_pp_trans,
	Perl_pp_sassign,
	Perl_pp_aassign,
	Perl_pp_chop,
	Perl_pp_schop,
	Perl_pp_chomp,
	Perl_pp_schomp,
	Perl_pp_defined,
	Perl_pp_undef,
	Perl_pp_study,
	Perl_pp_pos,
	Perl_pp_preinc,
	Perl_pp_i_preinc,
	Perl_pp_predec,
	Perl_pp_i_predec,
	Perl_pp_postinc,
	Perl_pp_i_postinc,
	Perl_pp_postdec,
	Perl_pp_i_postdec,
	Perl_pp_pow,
	Perl_pp_multiply,
	Perl_pp_i_multiply,
	Perl_pp_divide,
	Perl_pp_i_divide,
	Perl_pp_modulo,
	Perl_pp_i_modulo,
	Perl_pp_repeat,
	Perl_pp_add,
	Perl_pp_i_add,
	Perl_pp_subtract,
	Perl_pp_i_subtract,
	Perl_pp_concat,
	Perl_pp_stringify,
	Perl_pp_left_shift,
	Perl_pp_right_shift,
	Perl_pp_lt,
	Perl_pp_i_lt,
	Perl_pp_gt,
	Perl_pp_i_gt,
	Perl_pp_le,
	Perl_pp_i_le,
	Perl_pp_ge,
	Perl_pp_i_ge,
	Perl_pp_eq,
	Perl_pp_i_eq,
	Perl_pp_ne,
	Perl_pp_i_ne,
	Perl_pp_ncmp,
	Perl_pp_i_ncmp,
	Perl_pp_sle,	/* Perl_pp_slt */
	Perl_pp_sle,	/* Perl_pp_sgt */
	Perl_pp_sle,
	Perl_pp_sle,	/* Perl_pp_sge */
	Perl_pp_seq,
	Perl_pp_sne,
	Perl_pp_scmp,
	Perl_pp_bit_and,
	Perl_pp_bit_or,	/* Perl_pp_bit_xor */
	Perl_pp_bit_or,
	Perl_pp_negate,
	Perl_pp_i_negate,
	Perl_pp_not,
	Perl_pp_complement,
	Perl_pp_smartmatch,
	Perl_pp_atan2,
	Perl_pp_sin,
	Perl_pp_sin,	/* Perl_pp_cos */
	Perl_pp_rand,
	Perl_pp_srand,
	Perl_pp_sin,	/* Perl_pp_exp */
	Perl_pp_sin,	/* Perl_pp_log */
	Perl_pp_sin,	/* Perl_pp_sqrt */
	Perl_pp_int,
	Perl_pp_oct,	/* Perl_pp_hex */
	Perl_pp_oct,
	Perl_pp_abs,
	Perl_pp_length,
	Perl_pp_substr,
	Perl_pp_vec,
	Perl_pp_index,
	Perl_pp_index,	/* Perl_pp_rindex */
	Perl_pp_sprintf,
	Perl_pp_formline,
	Perl_pp_ord,
	Perl_pp_chr,
	Perl_pp_crypt,
	Perl_pp_ucfirst,
	Perl_pp_ucfirst,	/* Perl_pp_lcfirst */
	Perl_pp_uc,
	Perl_pp_lc,
	Perl_pp_quotemeta,
	Perl_pp_rv2av,
	Perl_pp_aelemfast,
	Perl_pp_aelem,
	Perl_pp_aslice,
	Perl_pp_aeach,
	Perl_pp_akeys,
	Perl_pp_akeys,	/* Perl_pp_avalues */
	Perl_pp_each,
	Perl_do_kv,	/* Perl_pp_values */
	Perl_do_kv,	/* Perl_pp_keys */
	Perl_pp_delete,
	Perl_pp_exists,
	Perl_pp_rv2av,	/* Perl_pp_rv2hv */
	Perl_pp_helem,
	Perl_pp_hslice,
	Perl_pp_boolkeys,
	Perl_pp_unpack,
	Perl_pp_pack,
	Perl_pp_split,
	Perl_pp_join,
	Perl_pp_list,
	Perl_pp_lslice,
	Perl_pp_anonlist,
	Perl_pp_anonhash,
	Perl_pp_splice,
	Perl_pp_push,
	Perl_pp_shift,	/* Perl_pp_pop */
	Perl_pp_shift,
	Perl_pp_unshift,
	Perl_pp_sort,
	Perl_pp_reverse,
	Perl_pp_grepstart,
	Perl_pp_grepwhile,
	Perl_unimplemented_op,	/* Perl_pp_mapstart */
	Perl_pp_mapwhile,
	Perl_pp_range,
	Perl_pp_flip,
	Perl_pp_flop,
	Perl_pp_and,
	Perl_pp_or,
	Perl_pp_xor,
	Perl_pp_defined,	/* Perl_pp_dor */
	Perl_pp_cond_expr,
	Perl_pp_and,	/* Perl_pp_andassign */
	Perl_pp_or,	/* Perl_pp_orassign */
	Perl_pp_defined,	/* Perl_pp_dorassign */
	Perl_pp_method,
	Perl_pp_entersub,
	Perl_pp_leavesub,
	Perl_pp_leavesublv,
	Perl_pp_caller,
	Perl_pp_warn,
	Perl_pp_die,
	Perl_pp_reset,
	Perl_pp_null,	/* Perl_pp_lineseq */
	Perl_pp_nextstate,
	Perl_pp_dbstate,
	Perl_pp_unstack,
	Perl_pp_enter,
	Perl_pp_leave,
	Perl_pp_null,	/* Perl_pp_scope */
	Perl_pp_enteriter,
	Perl_pp_iter,
	Perl_pp_enterloop,
	Perl_pp_leaveloop,
	Perl_pp_return,
	Perl_pp_last,
	Perl_pp_next,
	Perl_pp_redo,
	Perl_pp_goto,	/* Perl_pp_dump */
	Perl_pp_goto,
	Perl_pp_exit,
	Perl_pp_method_named,
	Perl_pp_entergiven,
	Perl_pp_leavegiven,
	Perl_pp_enterwhen,
	Perl_pp_leavewhen,
	Perl_pp_break,
	Perl_pp_continue,
	Perl_pp_open,
	Perl_pp_close,
	Perl_pp_pipe_op,
	Perl_pp_fileno,
	Perl_pp_umask,
	Perl_pp_binmode,
	Perl_pp_tie,
	Perl_pp_untie,
	Perl_pp_tied,
	Perl_pp_dbmopen,
	Perl_pp_untie,	/* Perl_pp_dbmclose */
	Perl_pp_sselect,
	Perl_pp_select,
	Perl_pp_getc,
	Perl_pp_sysread,	/* Perl_pp_read */
	Perl_pp_enterwrite,
	Perl_pp_leavewrite,
	Perl_pp_prtf,
	Perl_pp_print,
	Perl_pp_print,	/* Perl_pp_say */
	Perl_pp_sysopen,
	Perl_pp_sysseek,
	Perl_pp_sysread,
	Perl_pp_send,	/* Perl_pp_syswrite */
	Perl_pp_eof,
	Perl_pp_tell,
	Perl_pp_sysseek,	/* Perl_pp_seek */
	Perl_pp_truncate,
	Perl_pp_ioctl,	/* Perl_pp_fcntl */
	Perl_pp_ioctl,
	Perl_pp_flock,
	Perl_pp_send,
	Perl_pp_sysread,	/* Perl_pp_recv */
	Perl_pp_socket,
	Perl_pp_sockpair,
	Perl_pp_bind,
	Perl_pp_connect,
	Perl_pp_listen,
	Perl_pp_accept,
	Perl_pp_shutdown,
	Perl_pp_ssockopt,	/* Perl_pp_gsockopt */
	Perl_pp_ssockopt,
	Perl_pp_getpeername,	/* Perl_pp_getsockname */
	Perl_pp_getpeername,
	Perl_pp_stat,	/* Perl_pp_lstat */
	Perl_pp_stat,
	Perl_pp_ftrread,
	Perl_pp_ftrread,	/* Perl_pp_ftrwrite */
	Perl_pp_ftrread,	/* Perl_pp_ftrexec */
	Perl_pp_ftrread,	/* Perl_pp_fteread */
	Perl_pp_ftrread,	/* Perl_pp_ftewrite */
	Perl_pp_ftrread,	/* Perl_pp_fteexec */
	Perl_pp_ftis,
	Perl_pp_ftis,	/* Perl_pp_ftsize */
	Perl_pp_ftis,	/* Perl_pp_ftmtime */
	Perl_pp_ftis,	/* Perl_pp_ftatime */
	Perl_pp_ftis,	/* Perl_pp_ftctime */
	Perl_pp_ftrowned,
	Perl_pp_ftrowned,	/* Perl_pp_fteowned */
	Perl_pp_ftrowned,	/* Perl_pp_ftzero */
	Perl_pp_ftrowned,	/* Perl_pp_ftsock */
	Perl_pp_ftrowned,	/* Perl_pp_ftchr */
	Perl_pp_ftrowned,	/* Perl_pp_ftblk */
	Perl_pp_ftrowned,	/* Perl_pp_ftfile */
	Perl_pp_ftrowned,	/* Perl_pp_ftdir */
	Perl_pp_ftrowned,	/* Perl_pp_ftpipe */
	Perl_pp_ftrowned,	/* Perl_pp_ftsuid */
	Perl_pp_ftrowned,	/* Perl_pp_ftsgid */
	Perl_pp_ftrowned,	/* Perl_pp_ftsvtx */
	Perl_pp_ftlink,
	Perl_pp_fttty,
	Perl_pp_fttext,
	Perl_pp_fttext,	/* Perl_pp_ftbinary */
	Perl_pp_chdir,
	Perl_pp_chown,
	Perl_pp_chroot,
	Perl_pp_chown,	/* Perl_pp_unlink */
	Perl_pp_chown,	/* Perl_pp_chmod */
	Perl_pp_chown,	/* Perl_pp_utime */
	Perl_pp_rename,
	Perl_pp_link,
	Perl_pp_link,	/* Perl_pp_symlink */
	Perl_pp_readlink,
	Perl_pp_mkdir,
	Perl_pp_rmdir,
	Perl_pp_open_dir,
	Perl_pp_readdir,
	Perl_pp_telldir,
	Perl_pp_seekdir,
	Perl_pp_rewinddir,
	Perl_pp_closedir,
	Perl_pp_fork,
	Perl_pp_wait,
	Perl_pp_waitpid,
	Perl_pp_system,
	Perl_pp_exec,
	Perl_pp_chown,	/* Perl_pp_kill */
	Perl_pp_getppid,
	Perl_pp_getpgrp,
	Perl_pp_setpgrp,
	Perl_pp_getpriority,
	Perl_pp_setpriority,
	Perl_pp_time,
	Perl_pp_tms,
	Perl_pp_gmtime,	/* Perl_pp_localtime */
	Perl_pp_gmtime,
	Perl_pp_alarm,
	Perl_pp_sleep,
	Perl_pp_semget,	/* Perl_pp_shmget */
	Perl_pp_semctl,	/* Perl_pp_shmctl */
	Perl_pp_shmwrite,	/* Perl_pp_shmread */
	Perl_pp_shmwrite,
	Perl_pp_semget,	/* Perl_pp_msgget */
	Perl_pp_semctl,	/* Perl_pp_msgctl */
	Perl_pp_shmwrite,	/* Perl_pp_msgsnd */
	Perl_pp_shmwrite,	/* Perl_pp_msgrcv */
	Perl_pp_shmwrite,	/* Perl_pp_semop */
	Perl_pp_semget,
	Perl_pp_semctl,
	Perl_pp_require,
	Perl_pp_require,	/* Perl_pp_dofile */
	Perl_pp_hintseval,
	Perl_pp_entereval,
	Perl_pp_leaveeval,
	Perl_pp_entertry,
	Perl_pp_leavetry,
	Perl_pp_ghostent,	/* Perl_pp_ghbyname */
	Perl_pp_ghostent,	/* Perl_pp_ghbyaddr */
	Perl_pp_ghostent,
	Perl_pp_gnetent,	/* Perl_pp_gnbyname */
	Perl_pp_gnetent,	/* Perl_pp_gnbyaddr */
	Perl_pp_gnetent,
	Perl_pp_gprotoent,	/* Perl_pp_gpbyname */
	Perl_pp_gprotoent,	/* Perl_pp_gpbynumber */
	Perl_pp_gprotoent,
	Perl_pp_gservent,	/* Perl_pp_gsbyname */
	Perl_pp_gservent,	/* Perl_pp_gsbyport */
	Perl_pp_gservent,
	Perl_pp_shostent,
	Perl_pp_snetent,
	Perl_pp_sprotoent,
	Perl_pp_sservent,
	Perl_pp_ehostent,
	Perl_pp_enetent,
	Perl_pp_eprotoent,
	Perl_pp_eservent,
	Perl_pp_gpwent,	/* Perl_pp_gpwnam */
	Perl_pp_gpwent,	/* Perl_pp_gpwuid */
	Perl_pp_gpwent,
	Perl_pp_spwent,
	Perl_pp_epwent,
	Perl_pp_ggrent,	/* Perl_pp_ggrnam */
	Perl_pp_ggrent,	/* Perl_pp_ggrgid */
	Perl_pp_ggrent,
	Perl_pp_sgrent,
	Perl_pp_egrent,
	Perl_pp_getlogin,
	Perl_pp_syscall,
	Perl_pp_lock,
	Perl_pp_once,
	Perl_unimplemented_op,	/* Perl_pp_custom */
}
#endif
#ifdef PERL_PPADDR_INITED
;
#endif

#ifdef PERL_GLOBAL_STRUCT_INIT
#  define PERL_CHECK_INITED
static const Perl_check_t Gcheck[]
#else
#  ifndef PERL_GLOBAL_STRUCT
#    define PERL_CHECK_INITED
EXT Perl_check_t PL_check[] /* or perlvars.h */
#  endif
#endif
#if (defined(DOINIT) && !defined(PERL_GLOBAL_STRUCT)) || defined(PERL_GLOBAL_STRUCT_INIT)
#  define PERL_CHECK_INITED
= {
	Perl_ck_null,		/* null */
	Perl_ck_null,		/* stub */
	Perl_ck_fun,		/* scalar */
	Perl_ck_null,		/* pushmark */
	Perl_ck_null,		/* wantarray */
	Perl_ck_svconst,	/* const */
	Perl_ck_null,		/* gvsv */
	Perl_ck_null,		/* gv */
	Perl_ck_null,		/* gelem */
	Perl_ck_null,		/* padsv */
	Perl_ck_null,		/* padav */
	Perl_ck_null,		/* padhv */
	Perl_ck_null,		/* padany */
	Perl_ck_null,		/* pushre */
	Perl_ck_rvconst,	/* rv2gv */
	Perl_ck_rvconst,	/* rv2sv */
	Perl_ck_null,		/* av2arylen */
	Perl_ck_rvconst,	/* rv2cv */
	Perl_ck_anoncode,	/* anoncode */
	Perl_ck_null,		/* prototype */
	Perl_ck_spair,		/* refgen */
	Perl_ck_null,		/* srefgen */
	Perl_ck_fun,		/* ref */
	Perl_ck_fun,		/* bless */
	Perl_ck_open,		/* backtick */
	Perl_ck_glob,		/* glob */
	Perl_ck_readline,	/* readline */
	Perl_ck_null,		/* rcatline */
	Perl_ck_fun,		/* regcmaybe */
	Perl_ck_fun,		/* regcreset */
	Perl_ck_null,		/* regcomp */
	Perl_ck_match,		/* match */
	Perl_ck_match,		/* qr */
	Perl_ck_match,		/* subst */
	Perl_ck_null,		/* substcont */
	Perl_ck_match,		/* trans */
	Perl_ck_sassign,	/* sassign */
	Perl_ck_null,		/* aassign */
	Perl_ck_spair,		/* chop */
	Perl_ck_null,		/* schop */
	Perl_ck_spair,		/* chomp */
	Perl_ck_null,		/* schomp */
	Perl_ck_defined,	/* defined */
	Perl_ck_lfun,		/* undef */
	Perl_ck_fun,		/* study */
	Perl_ck_lfun,		/* pos */
	Perl_ck_lfun,		/* preinc */
	Perl_ck_lfun,		/* i_preinc */
	Perl_ck_lfun,		/* predec */
	Perl_ck_lfun,		/* i_predec */
	Perl_ck_lfun,		/* postinc */
	Perl_ck_lfun,		/* i_postinc */
	Perl_ck_lfun,		/* postdec */
	Perl_ck_lfun,		/* i_postdec */
	Perl_ck_null,		/* pow */
	Perl_ck_null,		/* multiply */
	Perl_ck_null,		/* i_multiply */
	Perl_ck_null,		/* divide */
	Perl_ck_null,		/* i_divide */
	Perl_ck_null,		/* modulo */
	Perl_ck_null,		/* i_modulo */
	Perl_ck_repeat,		/* repeat */
	Perl_ck_null,		/* add */
	Perl_ck_null,		/* i_add */
	Perl_ck_null,		/* subtract */
	Perl_ck_null,		/* i_subtract */
	Perl_ck_concat,		/* concat */
	Perl_ck_fun,		/* stringify */
	Perl_ck_bitop,		/* left_shift */
	Perl_ck_bitop,		/* right_shift */
	Perl_ck_null,		/* lt */
	Perl_ck_null,		/* i_lt */
	Perl_ck_null,		/* gt */
	Perl_ck_null,		/* i_gt */
	Perl_ck_null,		/* le */
	Perl_ck_null,		/* i_le */
	Perl_ck_null,		/* ge */
	Perl_ck_null,		/* i_ge */
	Perl_ck_null,		/* eq */
	Perl_ck_null,		/* i_eq */
	Perl_ck_null,		/* ne */
	Perl_ck_null,		/* i_ne */
	Perl_ck_null,		/* ncmp */
	Perl_ck_null,		/* i_ncmp */
	Perl_ck_null,		/* slt */
	Perl_ck_null,		/* sgt */
	Perl_ck_null,		/* sle */
	Perl_ck_null,		/* sge */
	Perl_ck_null,		/* seq */
	Perl_ck_null,		/* sne */
	Perl_ck_null,		/* scmp */
	Perl_ck_bitop,		/* bit_and */
	Perl_ck_bitop,		/* bit_xor */
	Perl_ck_bitop,		/* bit_or */
	Perl_ck_null,		/* negate */
	Perl_ck_null,		/* i_negate */
	Perl_ck_null,		/* not */
	Perl_ck_bitop,		/* complement */
	Perl_ck_smartmatch,	/* smartmatch */
	Perl_ck_fun,		/* atan2 */
	Perl_ck_fun,		/* sin */
	Perl_ck_fun,		/* cos */
	Perl_ck_fun,		/* rand */
	Perl_ck_fun,		/* srand */
	Perl_ck_fun,		/* exp */
	Perl_ck_fun,		/* log */
	Perl_ck_fun,		/* sqrt */
	Perl_ck_fun,		/* int */
	Perl_ck_fun,		/* hex */
	Perl_ck_fun,		/* oct */
	Perl_ck_fun,		/* abs */
	Perl_ck_fun,		/* length */
	Perl_ck_substr,		/* substr */
	Perl_ck_fun,		/* vec */
	Perl_ck_index,		/* index */
	Perl_ck_index,		/* rindex */
	Perl_ck_fun,		/* sprintf */
	Perl_ck_fun,		/* formline */
	Perl_ck_fun,		/* ord */
	Perl_ck_fun,		/* chr */
	Perl_ck_fun,		/* crypt */
	Perl_ck_fun,		/* ucfirst */
	Perl_ck_fun,		/* lcfirst */
	Perl_ck_fun,		/* uc */
	Perl_ck_fun,		/* lc */
	Perl_ck_fun,		/* quotemeta */
	Perl_ck_rvconst,	/* rv2av */
	Perl_ck_null,		/* aelemfast */
	Perl_ck_null,		/* aelem */
	Perl_ck_null,		/* aslice */
	Perl_ck_each,		/* aeach */
	Perl_ck_each,		/* akeys */
	Perl_ck_each,		/* avalues */
	Perl_ck_each,		/* each */
	Perl_ck_each,		/* values */
	Perl_ck_each,		/* keys */
	Perl_ck_delete,		/* delete */
	Perl_ck_exists,		/* exists */
	Perl_ck_rvconst,	/* rv2hv */
	Perl_ck_null,		/* helem */
	Perl_ck_null,		/* hslice */
	Perl_ck_fun,		/* boolkeys */
	Perl_ck_unpack,		/* unpack */
	Perl_ck_fun,		/* pack */
	Perl_ck_split,		/* split */
	Perl_ck_join,		/* join */
	Perl_ck_null,		/* list */
	Perl_ck_null,		/* lslice */
	Perl_ck_fun,		/* anonlist */
	Perl_ck_fun,		/* anonhash */
	Perl_ck_fun,		/* splice */
	Perl_ck_fun,		/* push */
	Perl_ck_shift,		/* pop */
	Perl_ck_shift,		/* shift */
	Perl_ck_fun,		/* unshift */
	Perl_ck_sort,		/* sort */
	Perl_ck_fun,		/* reverse */
	Perl_ck_grep,		/* grepstart */
	Perl_ck_null,		/* grepwhile */
	Perl_ck_grep,		/* mapstart */
	Perl_ck_null,		/* mapwhile */
	Perl_ck_null,		/* range */
	Perl_ck_null,		/* flip */
	Perl_ck_null,		/* flop */
	Perl_ck_null,		/* and */
	Perl_ck_null,		/* or */
	Perl_ck_null,		/* xor */
	Perl_ck_null,		/* dor */
	Perl_ck_null,		/* cond_expr */
	Perl_ck_null,		/* andassign */
	Perl_ck_null,		/* orassign */
	Perl_ck_null,		/* dorassign */
	Perl_ck_method,		/* method */
	Perl_ck_subr,		/* entersub */
	Perl_ck_null,		/* leavesub */
	Perl_ck_null,		/* leavesublv */
	Perl_ck_fun,		/* caller */
	Perl_ck_fun,		/* warn */
	Perl_ck_die,		/* die */
	Perl_ck_fun,		/* reset */
	Perl_ck_null,		/* lineseq */
	Perl_ck_null,		/* nextstate */
	Perl_ck_null,		/* dbstate */
	Perl_ck_null,		/* unstack */
	Perl_ck_null,		/* enter */
	Perl_ck_null,		/* leave */
	Perl_ck_null,		/* scope */
	Perl_ck_null,		/* enteriter */
	Perl_ck_null,		/* iter */
	Perl_ck_null,		/* enterloop */
	Perl_ck_null,		/* leaveloop */
	Perl_ck_return,		/* return */
	Perl_ck_null,		/* last */
	Perl_ck_null,		/* next */
	Perl_ck_null,		/* redo */
	Perl_ck_null,		/* dump */
	Perl_ck_null,		/* goto */
	Perl_ck_exit,		/* exit */
	Perl_ck_null,		/* method_named */
	Perl_ck_null,		/* entergiven */
	Perl_ck_null,		/* leavegiven */
	Perl_ck_null,		/* enterwhen */
	Perl_ck_null,		/* leavewhen */
	Perl_ck_null,		/* break */
	Perl_ck_null,		/* continue */
	Perl_ck_open,		/* open */
	Perl_ck_fun,		/* close */
	Perl_ck_fun,		/* pipe_op */
	Perl_ck_fun,		/* fileno */
	Perl_ck_fun,		/* umask */
	Perl_ck_fun,		/* binmode */
	Perl_ck_fun,		/* tie */
	Perl_ck_fun,		/* untie */
	Perl_ck_fun,		/* tied */
	Perl_ck_fun,		/* dbmopen */
	Perl_ck_fun,		/* dbmclose */
	Perl_ck_select,		/* sselect */
	Perl_ck_select,		/* select */
	Perl_ck_eof,		/* getc */
	Perl_ck_fun,		/* read */
	Perl_ck_fun,		/* enterwrite */
	Perl_ck_null,		/* leavewrite */
	Perl_ck_listiob,	/* prtf */
	Perl_ck_listiob,	/* print */
	Perl_ck_listiob,	/* say */
	Perl_ck_fun,		/* sysopen */
	Perl_ck_fun,		/* sysseek */
	Perl_ck_fun,		/* sysread */
	Perl_ck_fun,		/* syswrite */
	Perl_ck_eof,		/* eof */
	Perl_ck_fun,		/* tell */
	Perl_ck_fun,		/* seek */
	Perl_ck_trunc,		/* truncate */
	Perl_ck_fun,		/* fcntl */
	Perl_ck_fun,		/* ioctl */
	Perl_ck_fun,		/* flock */
	Perl_ck_fun,		/* send */
	Perl_ck_fun,		/* recv */
	Perl_ck_fun,		/* socket */
	Perl_ck_fun,		/* sockpair */
	Perl_ck_fun,		/* bind */
	Perl_ck_fun,		/* connect */
	Perl_ck_fun,		/* listen */
	Perl_ck_fun,		/* accept */
	Perl_ck_fun,		/* shutdown */
	Perl_ck_fun,		/* gsockopt */
	Perl_ck_fun,		/* ssockopt */
	Perl_ck_fun,		/* getsockname */
	Perl_ck_fun,		/* getpeername */
	Perl_ck_ftst,		/* lstat */
	Perl_ck_ftst,		/* stat */
	Perl_ck_ftst,		/* ftrread */
	Perl_ck_ftst,		/* ftrwrite */
	Perl_ck_ftst,		/* ftrexec */
	Perl_ck_ftst,		/* fteread */
	Perl_ck_ftst,		/* ftewrite */
	Perl_ck_ftst,		/* fteexec */
	Perl_ck_ftst,		/* ftis */
	Perl_ck_ftst,		/* ftsize */
	Perl_ck_ftst,		/* ftmtime */
	Perl_ck_ftst,		/* ftatime */
	Perl_ck_ftst,		/* ftctime */
	Perl_ck_ftst,		/* ftrowned */
	Perl_ck_ftst,		/* fteowned */
	Perl_ck_ftst,		/* ftzero */
	Perl_ck_ftst,		/* ftsock */
	Perl_ck_ftst,		/* ftchr */
	Perl_ck_ftst,		/* ftblk */
	Perl_ck_ftst,		/* ftfile */
	Perl_ck_ftst,		/* ftdir */
	Perl_ck_ftst,		/* ftpipe */
	Perl_ck_ftst,		/* ftsuid */
	Perl_ck_ftst,		/* ftsgid */
	Perl_ck_ftst,		/* ftsvtx */
	Perl_ck_ftst,		/* ftlink */
	Perl_ck_ftst,		/* fttty */
	Perl_ck_ftst,		/* fttext */
	Perl_ck_ftst,		/* ftbinary */
	Perl_ck_chdir,		/* chdir */
	Perl_ck_fun,		/* chown */
	Perl_ck_fun,		/* chroot */
	Perl_ck_fun,		/* unlink */
	Perl_ck_fun,		/* chmod */
	Perl_ck_fun,		/* utime */
	Perl_ck_fun,		/* rename */
	Perl_ck_fun,		/* link */
	Perl_ck_fun,		/* symlink */
	Perl_ck_fun,		/* readlink */
	Perl_ck_fun,		/* mkdir */
	Perl_ck_fun,		/* rmdir */
	Perl_ck_fun,		/* open_dir */
	Perl_ck_fun,		/* readdir */
	Perl_ck_fun,		/* telldir */
	Perl_ck_fun,		/* seekdir */
	Perl_ck_fun,		/* rewinddir */
	Perl_ck_fun,		/* closedir */
	Perl_ck_null,		/* fork */
	Perl_ck_null,		/* wait */
	Perl_ck_fun,		/* waitpid */
	Perl_ck_exec,		/* system */
	Perl_ck_exec,		/* exec */
	Perl_ck_fun,		/* kill */
	Perl_ck_null,		/* getppid */
	Perl_ck_fun,		/* getpgrp */
	Perl_ck_fun,		/* setpgrp */
	Perl_ck_fun,		/* getpriority */
	Perl_ck_fun,		/* setpriority */
	Perl_ck_null,		/* time */
	Perl_ck_null,		/* tms */
	Perl_ck_fun,		/* localtime */
	Perl_ck_fun,		/* gmtime */
	Perl_ck_fun,		/* alarm */
	Perl_ck_fun,		/* sleep */
	Perl_ck_fun,		/* shmget */
	Perl_ck_fun,		/* shmctl */
	Perl_ck_fun,		/* shmread */
	Perl_ck_fun,		/* shmwrite */
	Perl_ck_fun,		/* msgget */
	Perl_ck_fun,		/* msgctl */
	Perl_ck_fun,		/* msgsnd */
	Perl_ck_fun,		/* msgrcv */
	Perl_ck_fun,		/* semop */
	Perl_ck_fun,		/* semget */
	Perl_ck_fun,		/* semctl */
	Perl_ck_require,	/* require */
	Perl_ck_fun,		/* dofile */
	Perl_ck_svconst,	/* hintseval */
	Perl_ck_eval,		/* entereval */
	Perl_ck_null,		/* leaveeval */
	Perl_ck_eval,		/* entertry */
	Perl_ck_null,		/* leavetry */
	Perl_ck_fun,		/* ghbyname */
	Perl_ck_fun,		/* ghbyaddr */
	Perl_ck_null,		/* ghostent */
	Perl_ck_fun,		/* gnbyname */
	Perl_ck_fun,		/* gnbyaddr */
	Perl_ck_null,		/* gnetent */
	Perl_ck_fun,		/* gpbyname */
	Perl_ck_fun,		/* gpbynumber */
	Perl_ck_null,		/* gprotoent */
	Perl_ck_fun,		/* gsbyname */
	Perl_ck_fun,		/* gsbyport */
	Perl_ck_null,		/* gservent */
	Perl_ck_fun,		/* shostent */
	Perl_ck_fun,		/* snetent */
	Perl_ck_fun,		/* sprotoent */
	Perl_ck_fun,		/* sservent */
	Perl_ck_null,		/* ehostent */
	Perl_ck_null,		/* enetent */
	Perl_ck_null,		/* eprotoent */
	Perl_ck_null,		/* eservent */
	Perl_ck_fun,		/* gpwnam */
	Perl_ck_fun,		/* gpwuid */
	Perl_ck_null,		/* gpwent */
	Perl_ck_null,		/* spwent */
	Perl_ck_null,		/* epwent */
	Perl_ck_fun,		/* ggrnam */
	Perl_ck_fun,		/* ggrgid */
	Perl_ck_null,		/* ggrent */
	Perl_ck_null,		/* sgrent */
	Perl_ck_null,		/* egrent */
	Perl_ck_null,		/* getlogin */
	Perl_ck_fun,		/* syscall */
	Perl_ck_rfun,		/* lock */
	Perl_ck_null,		/* once */
	Perl_ck_null,		/* custom */
}
#endif
#ifdef PERL_CHECK_INITED
;
#endif /* #ifdef PERL_CHECK_INITED */

#ifndef PERL_GLOBAL_STRUCT_INIT

#ifndef DOINIT
EXTCONST U32 PL_opargs[];
#else
EXTCONST U32 PL_opargs[] = {
	0x00000000,	/* null */
	0x00000000,	/* stub */
	0x00001b04,	/* scalar */
	0x00000004,	/* pushmark */
	0x00000004,	/* wantarray */
	0x00000604,	/* const */
	0x00000644,	/* gvsv */
	0x00000644,	/* gv */
	0x00011240,	/* gelem */
	0x00000044,	/* padsv */
	0x00000040,	/* padav */
	0x00000040,	/* padhv */
	0x00000040,	/* padany */
	0x00000540,	/* pushre */
	0x00000144,	/* rv2gv */
	0x00000144,	/* rv2sv */
	0x00000104,	/* av2arylen */
	0x00000140,	/* rv2cv */
	0x00000600,	/* anoncode */
	0x00001b04,	/* prototype */
	0x00002101,	/* refgen */
	0x00001106,	/* srefgen */
	0x00009b8c,	/* ref */
	0x00091404,	/* bless */
	0x00009b88,	/* backtick */
	0x00009408,	/* glob */
	0x0000eb08,	/* readline */
	0x00000608,	/* rcatline */
	0x00001104,	/* regcmaybe */
	0x00001104,	/* regcreset */
	0x00001304,	/* regcomp */
	0x00000540,	/* match */
	0x00000504,	/* qr */
	0x00001544,	/* subst */
	0x00000344,	/* substcont */
	0x00001804,	/* trans */
	0x00000004,	/* sassign */
	0x00022208,	/* aassign */
	0x00002b0d,	/* chop */
	0x00009b8c,	/* schop */
	0x00002b1d,	/* chomp */
	0x00009b9c,	/* schomp */
	0x00009b84,	/* defined */
	0x00009b04,	/* undef */
	0x00009b84,	/* study */
	0x00009b8c,	/* pos */
	0x00001164,	/* preinc */
	0x00001144,	/* i_preinc */
	0x00001164,	/* predec */
	0x00001144,	/* i_predec */
	0x0000116c,	/* postinc */
	0x0000115c,	/* i_postinc */
	0x0000116c,	/* postdec */
	0x0000115c,	/* i_postdec */
	0x0001121e,	/* pow */
	0x0001123e,	/* multiply */
	0x0001121e,	/* i_multiply */
	0x0001123e,	/* divide */
	0x0001121e,	/* i_divide */
	0x0001123e,	/* modulo */
	0x0001121e,	/* i_modulo */
	0x00012209,	/* repeat */
	0x0001123e,	/* add */
	0x0001121e,	/* i_add */
	0x0001123e,	/* subtract */
	0x0001121e,	/* i_subtract */
	0x0001121e,	/* concat */
	0x0000141e,	/* stringify */
	0x0001121e,	/* left_shift */
	0x0001121e,	/* right_shift */
	0x00011226,	/* lt */
	0x00011206,	/* i_lt */
	0x00011226,	/* gt */
	0x00011206,	/* i_gt */
	0x00011226,	/* le */
	0x00011206,	/* i_le */
	0x00011226,	/* ge */
	0x00011206,	/* i_ge */
	0x00011226,	/* eq */
	0x00011206,	/* i_eq */
	0x00011226,	/* ne */
	0x00011206,	/* i_ne */
	0x0001122e,	/* ncmp */
	0x0001120e,	/* i_ncmp */
	0x00011206,	/* slt */
	0x00011206,	/* sgt */
	0x00011206,	/* sle */
	0x00011206,	/* sge */
	0x00011206,	/* seq */
	0x00011206,	/* sne */
	0x0001120e,	/* scmp */
	0x0001120e,	/* bit_and */
	0x0001120e,	/* bit_xor */
	0x0001120e,	/* bit_or */
	0x0000112e,	/* negate */
	0x0000111e,	/* i_negate */
	0x00001106,	/* not */
	0x0000110e,	/* complement */
	0x00000204,	/* smartmatch */
	0x0001141e,	/* atan2 */
	0x00009b9e,	/* sin */
	0x00009b9e,	/* cos */
	0x00009b1c,	/* rand */
	0x00009b1c,	/* srand */
	0x00009b9e,	/* exp */
	0x00009b9e,	/* log */
	0x00009b9e,	/* sqrt */
	0x00009b9e,	/* int */
	0x00009b9e,	/* hex */
	0x00009b9e,	/* oct */
	0x00009b9e,	/* abs */
	0x00009b9e,	/* length */
	0x0991140c,	/* substr */
	0x0011140c,	/* vec */
	0x0091141c,	/* index */
	0x0091141c,	/* rindex */
	0x0002140f,	/* sprintf */
	0x00021405,	/* formline */
	0x00009b9e,	/* ord */
	0x00009b9e,	/* chr */
	0x0001141e,	/* crypt */
	0x00009b8e,	/* ucfirst */
	0x00009b8e,	/* lcfirst */
	0x00009b8e,	/* uc */
	0x00009b8e,	/* lc */
	0x00009b8e,	/* quotemeta */
	0x00000148,	/* rv2av */
	0x00013604,	/* aelemfast */
	0x00013204,	/* aelem */
	0x00023401,	/* aslice */
	0x00003b00,	/* aeach */
	0x00003b08,	/* akeys */
	0x00003b08,	/* avalues */
	0x00004b00,	/* each */
	0x00004b08,	/* values */
	0x00004b08,	/* keys */
	0x00001b00,	/* delete */
	0x00001b04,	/* exists */
	0x00000148,	/* rv2hv */
	0x00014204,	/* helem */
	0x00024401,	/* hslice */
	0x00004b00,	/* boolkeys */
	0x00091400,	/* unpack */
	0x0002140d,	/* pack */
	0x00111408,	/* split */
	0x0002140d,	/* join */
	0x00002401,	/* list */
	0x00224200,	/* lslice */
	0x00002405,	/* anonlist */
	0x00002405,	/* anonhash */
	0x02993401,	/* splice */
	0x0002341d,	/* push */
	0x0000bb04,	/* pop */
	0x0000bb04,	/* shift */
	0x0002341d,	/* unshift */
	0x0002d441,	/* sort */
	0x00002409,	/* reverse */
	0x00025441,	/* grepstart */
	0x00000348,	/* grepwhile */
	0x00025441,	/* mapstart */
	0x00000348,	/* mapwhile */
	0x00011300,	/* range */
	0x00011100,	/* flip */
	0x00000100,	/* flop */
	0x00000300,	/* and */
	0x00000300,	/* or */
	0x00011206,	/* xor */
	0x00000300,	/* dor */
	0x00000340,	/* cond_expr */
	0x00000304,	/* andassign */
	0x00000304,	/* orassign */
	0x00000304,	/* dorassign */
	0x00000140,	/* method */
	0x00002149,	/* entersub */
	0x00000100,	/* leavesub */
	0x00000100,	/* leavesublv */
	0x00009b08,	/* caller */
	0x0000240d,	/* warn */
	0x0000244d,	/* die */
	0x00009b04,	/* reset */
	0x00000400,	/* lineseq */
	0x00000a04,	/* nextstate */
	0x00000a04,	/* dbstate */
	0x00000004,	/* unstack */
	0x00000000,	/* enter */
	0x00000400,	/* leave */
	0x00000400,	/* scope */
	0x00000940,	/* enteriter */
	0x00000000,	/* iter */
	0x00000940,	/* enterloop */
	0x00000200,	/* leaveloop */
	0x00002441,	/* return */
	0x00000d44,	/* last */
	0x00000d44,	/* next */
	0x00000d44,	/* redo */
	0x00000d44,	/* dump */
	0x00000d44,	/* goto */
	0x00009b44,	/* exit */
	0x00000640,	/* method_named */
	0x00000340,	/* entergiven */
	0x00000100,	/* leavegiven */
	0x00000340,	/* enterwhen */
	0x00000100,	/* leavewhen */
	0x00000000,	/* break */
	0x00000000,	/* continue */
	0x0029640d,	/* open */
	0x0000eb04,	/* close */
	0x00066404,	/* pipe_op */
	0x00006b0c,	/* fileno */
	0x00009b0c,	/* umask */
	0x00096404,	/* binmode */
	0x00217445,	/* tie */
	0x00007b04,	/* untie */
	0x00007b04,	/* tied */
	0x00114404,	/* dbmopen */
	0x00004b04,	/* dbmclose */
	0x01111408,	/* sselect */
	0x0000e40c,	/* select */
	0x0000eb0c,	/* getc */
	0x0917640d,	/* read */
	0x0000eb44,	/* enterwrite */
	0x00000100,	/* leavewrite */
	0x0002e405,	/* prtf */
	0x0002e405,	/* print */
	0x0002e405,	/* say */
	0x09116404,	/* sysopen */
	0x00116404,	/* sysseek */
	0x0917640d,	/* sysread */
	0x0991640d,	/* syswrite */
	0x0000eb04,	/* eof */
	0x0000eb0c,	/* tell */
	0x00116404,	/* seek */
	0x00011404,	/* truncate */
	0x0011640c,	/* fcntl */
	0x0011640c,	/* ioctl */
	0x0001641c,	/* flock */
	0x0911640d,	/* send */
	0x0117640d,	/* recv */
	0x01116404,	/* socket */
	0x11166404,	/* sockpair */
	0x00016404,	/* bind */
	0x00016404,	/* connect */
	0x00016404,	/* listen */
	0x0006640c,	/* accept */
	0x0001640c,	/* shutdown */
	0x00116404,	/* gsockopt */
	0x01116404,	/* ssockopt */
	0x00006b04,	/* getsockname */
	0x00006b04,	/* getpeername */
	0x00006c80,	/* lstat */
	0x00006c80,	/* stat */
	0x00006c84,	/* ftrread */
	0x00006c84,	/* ftrwrite */
	0x00006c84,	/* ftrexec */
	0x00006c84,	/* fteread */
	0x00006c84,	/* ftewrite */
	0x00006c84,	/* fteexec */
	0x00006c84,	/* ftis */
	0x00006c8c,	/* ftsize */
	0x00006c8c,	/* ftmtime */
	0x00006c8c,	/* ftatime */
	0x00006c8c,	/* ftctime */
	0x00006c84,	/* ftrowned */
	0x00006c84,	/* fteowned */
	0x00006c84,	/* ftzero */
	0x00006c84,	/* ftsock */
	0x00006c84,	/* ftchr */
	0x00006c84,	/* ftblk */
	0x00006c84,	/* ftfile */
	0x00006c84,	/* ftdir */
	0x00006c84,	/* ftpipe */
	0x00006c84,	/* ftsuid */
	0x00006c84,	/* ftsgid */
	0x00006c84,	/* ftsvtx */
	0x00006c84,	/* ftlink */
	0x00006c04,	/* fttty */
	0x00006c84,	/* fttext */
	0x00006c84,	/* ftbinary */
	0x00009b1c,	/* chdir */
	0x0000241d,	/* chown */
	0x00009b9c,	/* chroot */
	0x0000249d,	/* unlink */
	0x0000241d,	/* chmod */
	0x0000241d,	/* utime */
	0x0001141c,	/* rename */
	0x0001141c,	/* link */
	0x0001141c,	/* symlink */
	0x00009b8c,	/* readlink */
	0x0009949c,	/* mkdir */
	0x00009b9c,	/* rmdir */
	0x00016404,	/* open_dir */
	0x00006b00,	/* readdir */
	0x00006b0c,	/* telldir */
	0x00016404,	/* seekdir */
	0x00006b04,	/* rewinddir */
	0x00006b04,	/* closedir */
	0x0000000c,	/* fork */
	0x0000001c,	/* wait */
	0x0001141c,	/* waitpid */
	0x0002941d,	/* system */
	0x0002945d,	/* exec */
	0x0000245d,	/* kill */
	0x0000001c,	/* getppid */
	0x00009b1c,	/* getpgrp */
	0x0009941c,	/* setpgrp */
	0x0001141c,	/* getpriority */
	0x0011141c,	/* setpriority */
	0x0000001c,	/* time */
	0x00000000,	/* tms */
	0x00009b08,	/* localtime */
	0x00009b08,	/* gmtime */
	0x00009b8c,	/* alarm */
	0x00009b1c,	/* sleep */
	0x0011140d,	/* shmget */
	0x0011140d,	/* shmctl */
	0x0111140d,	/* shmread */
	0x0111140d,	/* shmwrite */
	0x0001140d,	/* msgget */
	0x0011140d,	/* msgctl */
	0x0011140d,	/* msgsnd */
	0x1111140d,	/* msgrcv */
	0x0001140d,	/* semop */
	0x0011140d,	/* semget */
	0x0111140d,	/* semctl */
	0x00009bc0,	/* require */
	0x00001140,	/* dofile */
	0x00000604,	/* hintseval */
	0x00001b40,	/* entereval */
	0x00001100,	/* leaveeval */
	0x00000b40,	/* entertry */
	0x00000400,	/* leavetry */
	0x00001b00,	/* ghbyname */
	0x00011400,	/* ghbyaddr */
	0x00000000,	/* ghostent */
	0x00001b00,	/* gnbyname */
	0x00011400,	/* gnbyaddr */
	0x00000000,	/* gnetent */
	0x00001b00,	/* gpbyname */
	0x00001400,	/* gpbynumber */
	0x00000000,	/* gprotoent */
	0x00011400,	/* gsbyname */
	0x00011400,	/* gsbyport */
	0x00000000,	/* gservent */
	0x00001b04,	/* shostent */
	0x00001b04,	/* snetent */
	0x00001b04,	/* sprotoent */
	0x00001b04,	/* sservent */
	0x00000004,	/* ehostent */
	0x00000004,	/* enetent */
	0x00000004,	/* eprotoent */
	0x00000004,	/* eservent */
	0x00001b00,	/* gpwnam */
	0x00001b00,	/* gpwuid */
	0x00000000,	/* gpwent */
	0x00000004,	/* spwent */
	0x00000004,	/* epwent */
	0x00001b00,	/* ggrnam */
	0x00001b00,	/* ggrgid */
	0x00000000,	/* ggrent */
	0x00000004,	/* sgrent */
	0x00000004,	/* egrent */
	0x0000000c,	/* getlogin */
	0x0002140d,	/* syscall */
	0x00007b04,	/* lock */
	0x00000300,	/* once */
	0x00000000,	/* custom */
};
#endif

#endif /* !PERL_GLOBAL_STRUCT_INIT */

END_EXTERN_C

/* ex: set ro: */
