#! perl -w
#
# $Id$
#
# This script is written intentionally in EUC-JP
# -- dankogai

BEGIN {
    require Config; import Config;
    if ($Config{'extensions'} !~ /\bEncode\b/) {
      print "1..0 # Skip: Encode was not built\n";
      exit 0;
    }
    if (ord("A") == 193) {
        print "1..0 # Skip: EBCDIC\n";
        exit 0;
    }
    unless (PerlIO::Layer->find('perlio')){
        print "1..0 # Skip: PerlIO required\n";
        exit 0;
    }
    if ($ENV{PERL_CORE_MINITEST}) {
        print "1..0 # Skip: no dynamic loading on miniperl, no Encode\n";
        exit 0;
    }
    $| = 1;
    require './test.pl';
}

use strict;
plan(tests => 6);
use encoding 'euc-jp';

my @hiragana =  map {chr} ord("¤¡")..ord("¤ó");
my @katakana =  map {chr} ord("¥¡")..ord("¥ó");
my $hiragana = join('' => @hiragana);
my $katakana = join('' => @katakana);
my %h2k; @h2k{@hiragana} = @katakana;
my %k2h; @k2h{@katakana} = @hiragana;

# print @hiragana, "\n";

my $str;

$str = $hiragana; $str =~ tr/¤¡-¤ó/¥¡-¥ó/;
is($str, $katakana, "tr// # hiragana -> katakana");
$str = $katakana; $str =~ tr/¥¡-¥ó/¤¡-¤ó/;
is($str, $hiragana, "tr// # hiragana -> katakana");

$str = $hiragana; eval qq(\$str =~ tr/¤¡-¤ó/¥¡-¥ó/);
is($str, $katakana, "eval qq(tr//) # hiragana -> katakana");
$str = $katakana; eval qq(\$str =~ tr/¥¡-¥ó/¤¡-¤ó/);
is($str, $hiragana, "eval qq(tr//) # hiragana -> katakana");

$str = $hiragana; $str =~ s/([¤¡-¤ó])/$h2k{$1}/go;
is($str, $katakana, "s/// # hiragana -> katakana");
$str = $katakana; $str =~ s/([¥¡-¥ó])/$k2h{$1}/go;
is($str, $hiragana, "s/// # hiragana -> katakana");
__END__
